require 'httparty'
require 'logger'
require 'thread'

module LL
  module WK
    module API
      module Connection
        class HTTParty < Base
          include ::HTTParty
          ::LL::WK::API::Connection.register('httparty', self)

          def self.authenticate(email, password, url)
            base_uri url
            resp = post('/session/new',
                        body: authenticate_payload(email, password),
                        headers: { 'Content-Type' => 'application/json' })
            yield(resp) if block_given?
            resp
          end

          def initialize(url:, email:, password:)
            self.class.base_uri url
            super(url: url, email: email, password: password)
          end


          def response_from_api(endpoint, params)
            self.class.base_uri url
            resp = self.class.get(sanitize_endpoint(endpoint), query: params, headers: { 'Authorization' => auth })
            self.class.trap_resp_code(resp.code)
            yield(resp) if block_given?
            resp
          end

          private

          def sanitize_endpoint(endpoint)
            File.join('', endpoint)
          end
        end
      end
    end
  end
end
