require 'curb'
require 'json'
require 'addressable/uri'

class Hash
  def to_query
    uri = Addressable::URI.new
    uri.query_values = self
    uri.query
  end
end

module LL
  module WK
    module API
      module Connection
        class Curb < Base
          ::LL::WK::API::Connection.register('curb', self)

          def self.autenticate(email, password, url)
            curl_easy("#{url}/session/new") do |curl|
              curl.headers['Content-Type'] = 'application/json'
              curl.http_post(authenticate_payload(email, password))
              resp = Json.parse(curl.body_str)
              self.class.trap_resp_code(resp['status'])
              return resp
            end
          end

          def response_from_api(endpoint, params)
            curl_easy("#{url}/#{endpoint}?#{params.to_query}") do |curl|
              curl.perform
              resp = JSON.parse(curl.body_str)
              self.class.trap_resp_code(curl.status)
              yield(resp) if block_given?
              return resp
            end
          end

          private

          def curl_easy(uri)
            Curl::Easy.new(uri) do |c|
              c.follow_location = true
              c.headers['Authorization'] = auth unless token_expired?
              yield(c) if block_given?
            end
          end
        end
      end
    end
  end
end
